<?php

declare(strict_types=1);

namespace Drupal\commerce_cart_redirection\Hook;

use Drupal\Component\Render\PlainTextOutput;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Security\TrustedCallbackInterface;

class Hooks implements TrustedCallbackInterface {

  /**
   * Alters forms to modify the configuration and behavior based on product settings.
   *
   * This hook implementation modifies the "Add to Cart" button text on forms
   * based on specific product variation types and configured settings from the
   * module.
   *
   * @param array $form
   *   The form array being altered.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param string|null $form_id
   *   (optional) The unique string identifying the form. Defaults to NULL.
   *
   * @return void
   *   This method does not return a value.
   */
  #[Hook('form_alter')]
  public function formAlter(&$form, FormStateInterface $form_state, $form_id = NULL) {
    if ($product = $form_state?->get('product')) {
      // Load module config and see if there is text in the
      // 'Modify Add to Cart button text' field. Skip this whole thing if
      // there is not.
      $config = \Drupal::configFactory()?->get('commerce_cart_redirection.settings');
      if (!empty($config?->get('add_to_cart_replacement_text'))) {
        // Load the Product Variation types this module is redirecting.
        // If the current variation is one of them then go ahead and
        // change the button text.
        $active_bundles = $config->get('product_bundles');
        $negate = $config?->get('negate_product_bundles');
        $redirect = FALSE;

        if ($purchasable_bundle = $form_state->getFormObject()
          ->getEntity()
          ->get('purchased_entity')
          ?->referencedEntities()[0]?->bundle()) {
          if (isset($active_bundles[$purchasable_bundle]) && $active_bundles[$purchasable_bundle] !== 0) {
            if (!$negate) {
              $redirect = TRUE;
            }
          }
          else {
            if ($negate) {
              $redirect = TRUE;
            }
          }
          if ($redirect) {
            $form['actions']['submit']['#post_render'] = [
              [$this, 'modifyAtcButtonText']
            ];
          }
        }
      }
    }
  }

  /**
   * Registers and returns a list of trusted callback methods.
   *
   * @return array An array of callback method names.
   */
  public static function trustedCallbacks(): array {
    return ['modifyAtcButtonText'];
  }

  /**
   * Modifies the "Add to cart" button text in the provided HTML.
   *
   * @param string $html The original HTML containing the button text to be modified.
   * @param array $element Additional element data that may be used in modification.
   *
   * @return string The modified HTML with the button text replaced.
   */
  public function modifyAtcButtonText(string $html, array $element){
    $search_text = PlainTextOutput::renderFromHtml(t('Add to cart'));
    return str_replace($search_text, self::replacementText($search_text), $html);
  }

  /**
   * Retrieves the replacement text for the "Add to Cart" button.
   *
   * @param string $original
   *   The original button text.
   *
   * @return string
   *   The replacement text if configured; otherwise, the original text.
   */
  private static function replacementText(string $original):string {
    $config = \Drupal::configFactory()->get('commerce_cart_redirection.settings');
    return $config->get('add_to_cart_replacement_text') ?? $original;
  }


  #[Hook('help')]
  public function help($route_name, RouteMatchInterface $route_match) {
    switch ($route_name) {
      case 'help.page.commerce_cart_redirection':
        $output = Markup::create(
          '<h3>' . t('About') . '</h3>' .
          '<p>' . t('Redirect to your cart, checkout, or other URL when a user adds a selected product variation bundle to their cart.') . '</p>'
        );
        return $output;
    }
  }

}
